@@ -20,7 +20,7 @@
 //! During the countdown, the program will react to Ctrl-C/SIGINT/SIGTERM and will cancel the countdown task accordingly.
 //!
 //! ```
-//! use anyhow::Result;
+//! use miette::Result;
 //! use tokio_graceful_shutdown::{SubsystemHandle, Toplevel};
 //! use env_logger::{Builder, Env};
 //! use tokio::time::{sleep, Duration};
@@ -82,6 +82,8 @@
 //! to initiate a shutdown.
 //!
 
+#![deny(missing_docs)]
+#![deny(rustdoc::missing_doc_code_examples)]
 #![doc(
     issue_tracker_base_url = "https://github.com/Finomnis/tokio-graceful-shutdown/issues",
     test(no_crate_inject, attr(deny(warnings))),
@@ -91,17 +93,20 @@
 type BoxedError = Box<dyn std::error::Error + Send + Sync + 'static>;
 
 mod errors;
-mod event;
 mod exit_state;
+mod into_subsystem;
 mod runner;
 mod shutdown_token;
 mod signal_handling;
 mod subsystem;
 mod toplevel;
+mod utils;
 
 pub use errors::GracefulShutdownError;
+pub use errors::PartialShutdownError;
+pub use errors::SubsystemError;
+pub use into_subsystem::IntoSubsystem;
 pub use shutdown_token::ShutdownToken;
 pub use subsystem::NestedSubsystem;
-pub use subsystem::PartialShutdownError;
 pub use subsystem::SubsystemHandle;
 pub use toplevel::Toplevel;
